@@ -33,10 +33,10 @@ class Gridder2D(Gridder):
     def __init__(self,nx,ny):
         Gridder.__init__(self)
 
-        self.xmin = 0
-        self.ymin = 0
-        self.xmax = 0
-        self.ymax = 0
+        self.xmin = None
+        self.ymin = None
+        self.xmax = None
+        self.ymax = None
 
         self.nx = nx
         self.ny = ny
@@ -49,10 +49,10 @@ class Gridder2D(Gridder):
         class attributes.
         """
         
-        self.gdata = numpy.zeros((self.nx,self.ny),dtype=numpy.double)
-        self.gdata = check_array(self.gdata,numpy.double)
-        self.gnorm = numpy.zeros((self.nx,self.ny),dtype=numpy.double)
-        self.gnorm = check_array(self.gnorm,numpy.double)
+        self._gdata = numpy.zeros((self.nx,self.ny),dtype=numpy.double)
+        self._gdata = check_array(self._gdata,numpy.double)
+        self._gnorm = numpy.zeros((self.nx,self.ny),dtype=numpy.double)
+        self._gnorm = check_array(self._gnorm,numpy.double)
 
 
     def SetResolution(self,nx,ny):
@@ -82,22 +82,10 @@ class Gridder2D(Gridder):
     def __get_ymatrix(self):
         return ones(self.nx,self.ny)*self.yaxis[numpy.newaxis,:]
 
-    def __get_data(self):
-        """
-        return gridded data (performs normalization if switched on)
-        """
-        if self.normalize:
-            tmp= numpy.copy(self.gdata)
-            tmp[self.gnorm!=0] /= self.gnorm[self.gnorm!=0].astype(numpy.float)
-            return tmp
-        else:
-            return self.gdata.copy()
-
     yaxis = property(__get_yaxis)
     xaxis = property(__get_xaxis)
     xmatrix = property(__get_xmatrix)
     ymatrix = property(__get_ymatrix)
-    data = property(__get_data)
 
     def dataRange(self,(xmin,xmax),(ymin,ymax),fixed=True):
         """
@@ -138,7 +126,6 @@ class Gridder2D(Gridder):
         y = args[1]
         data = args[2]
 
-
         if isinstance(x,(list,tuple,numpy.float,numpy.int)):
             x = numpy.array(x) 
         if isinstance(y,(list,tuple,numpy.float,numpy.int)):
@@ -153,12 +140,11 @@ class Gridder2D(Gridder):
         if x.size != y.size or y.size!=data.size:
             raise exception.InputError("XU.Gridder2D: size of given datasets (x,y,data) is not equal!")
  
-        if not self.fixed_range:
-            self.xmin = x.min()
-            self.xmax = x.max()
-            self.ymin = y.min()
-            self.ymax = y.max()
-        
+        if not self.fixed_range: 
+            # assume that with setting keep_data the user wants to call the gridder
+            # more often and obtain a reasonable result
+            self.dataRange((x.min(),x.max()),(y.min(),y.max()),self.keep_data)
+
         # require correct aligned memory for input arrays
         lx = check_array(x,numpy.double)
         ly = check_array(y,numpy.double)
@@ -169,5 +155,5 @@ class Gridder2D(Gridder):
         cxrayutilities.gridder2d(lx,ly,ldata,self.nx,self.ny,
                                  self.xmin,self.xmax,
                                  self.ymin,self.ymax,
-                                 self.gdata,self.gnorm,flags)            
+                                 self._gdata,self._gnorm,flags)            
 
