@@ -51,10 +51,10 @@ class Gridder3D(Gridder):
         Class method to allocate memory for the gridder based on the nx,ny 
         class attributes.
         """
-        self.gdata = numpy.zeros((self.nx,self.ny,self.nz),dtype=numpy.double)
-        self.gdata = check_array(self.gdata,numpy.double)
-        self.gnorm = numpy.zeros((self.nx,self.ny,self.nz),dtype=numpy.double)
-        self.gnorm = check_array(self.gnorm,numpy.double)
+        self._gdata = numpy.zeros((self.nx,self.ny,self.nz),dtype=numpy.double)
+        self._gdata = check_array(self._gdata,numpy.double)
+        self._gnorm = numpy.zeros((self.nx,self.ny,self.nz),dtype=numpy.double)
+        self._gnorm = check_array(self._gnorm,numpy.double)
 
     def SetResolution(self,nx,ny,nz):
         self.nx = nx
@@ -84,24 +84,12 @@ class Gridder3D(Gridder):
         return ones(self.nx,self.ny,self.nz)*\
                 self.zaxis[numpy.newaxis,numpy.newaxis,:]
         
-    def __get_data(self):
-        """
-        return gridded data (performs normalization if switched on)
-        """
-        if self.normalize:
-            tmp= numpy.copy(self.gdata)
-            tmp[self.gnorm!=0] /= self.gnorm[self.gnorm!=0].astype(numpy.float)
-            return tmp
-        else:
-            return self.gdata.copy()
-
     zaxis = property(__get_zaxis)
     zmatrix = property(__get_zmatrix)
     xaxis = property(__get_xaxis)
     xmatrix = property(__get_xmatrix)
     yaxis = property(__get_yaxis)
     ymatrix = property(__get_ymatrix)
-    data = property(__get_data)
 
     def dataRange(self,(xmin,xmax),(ymin,ymax),(zmin,zmax),fixed=True):
         """
@@ -158,13 +146,10 @@ class Gridder3D(Gridder):
         if x.size != y.size or y.size!=z.size or z.size!=data.size:
             raise exception.InputError("XU.Gridder3D: size of given datasets (x,y,z,data) is not equal!")
 
-        if not self.fixed_range:
-            self.xmin = x.min()
-            self.xmax = x.max()
-            self.ymin = y.min()
-            self.ymax = y.max()
-            self.zmin = z.min()
-            self.zmax = z.max()
+        if not self.fixed_range: 
+            # assume that with setting keep_data the user wants to call the gridder
+            # more often and obtain a reasonable result
+            self.dataRange((x.min(),x.max()),(y.min(),y.max()),(z.min(),z.max()),self.keep_data)
         
         # require correct aligned memory for input arrays
         lx = check_array(x,numpy.double)
@@ -178,5 +163,5 @@ class Gridder3D(Gridder):
                                  self.xmin,self.xmax,
                                  self.ymin,self.ymax,
                                  self.zmin,self.zmax,
-                                 self.gdata,self.gnorm,flags)            
+                                 self._gdata,self._gnorm,flags)            
 
